'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var readline = require('readline');
var execSync = require('child_process').execSync;
var chalk = require('chalk');
var ansiEscapes = require('ansi-escapes');
var debounce = require('debounce-promise');
var gitFiles = require('git-files');
var gitDiffGlob = require('git-diff-glob');
var jumper = require('terminal-jumper');
var pager = require('node-pager');
var utils = require('./utils');
require('./utils/readline-hack');

var Interactive = function () {
	function Interactive() {
		var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

		_classCallCheck(this, Interactive);

		this.options = options;
		this.fileIndex = 0;
		this.showingIndicator = false;
		this.showingDiff = false;

		this.instructions = chalk.gray('Arrow keys to navigate') + '\n' + chalk.gray('Tab to show diff of selected file(s)');
		this.gitAddPrompt = '' + chalk.green('Enter a file glob: ');

		this.render = debounce(this.render.bind(this), 200);
		this.findGitFiles = debounce(this.findGitFiles.bind(this), 200);

		this.onType = this.onType.bind(this);
		this.onTab = this.onTab.bind(this);
		this.onArrow = this.onArrow.bind(this);
		this.exitBeforeAdd = this.exitBeforeAdd.bind(this);
		this.exitAfterAdd = this.exitAfterAdd.bind(this);

		process.on('exit', this.exitBeforeAdd);
	}

	_createClass(Interactive, [{
		key: 'run',
		value: function run() {
			readline.emitKeypressEvents(process.stdin);
			this.rl = readline.createInterface({
				input: process.stdin,
				output: process.stdout,
				terminal: true,
				prompt: '',
				historySize: 0
			});

			var stagedFiles = gitFiles.staged();

			if (stagedFiles.length > 0) {
				this.showAlreadyAddedFilesWarning(stagedFiles);
			} else {
				this.showGlobPrompt();
			}
		}
	}, {
		key: 'showAlreadyAddedFilesWarning',
		value: function showAlreadyAddedFilesWarning(stagedFiles) {
			var _this = this;

			jumper.block('There are file(s) already staged for commit.');
			var _iteratorNormalCompletion = true;
			var _didIteratorError = false;
			var _iteratorError = undefined;

			try {
				for (var _iterator = stagedFiles[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
					var file = _step.value;

					jumper.block(chalk.red(file));
				}
			} catch (err) {
				_didIteratorError = true;
				_iteratorError = err;
			} finally {
				try {
					if (!_iteratorNormalCompletion && _iterator.return) {
						_iterator.return();
					}
				} finally {
					if (_didIteratorError) {
						throw _iteratorError;
					}
				}
			}

			jumper.break();
			jumper.block(chalk.bold('These file(s) will be committed. Continue? (y/n) '), 'warning');

			this.render();
			jumper.jumpTo('warning', -1);

			this.rl.once('line', function (line) {
				var answer = line.trim().toLowerCase();
				jumper.erase();

				if (['y', 'yes'].includes(answer)) {
					jumper.reset();
					_this.showGlobPrompt();
				} else {
					process.exit();
				}
			});
		}
	}, {
		key: 'showGlobPrompt',
		value: function showGlobPrompt() {
			var _this2 = this;

			this.rl.setPrompt(this.gitAddPrompt);

			// show prompt and available files
			jumper.block(this.instructions, 'instructions');
			jumper.break();
			jumper.block(this.gitAddPrompt, 'enter');
			jumper.break();
			jumper.block(chalk.green('Files found:'), 'found');

			this.findGitFiles().then(function () {
				jumper.render();
				_this2.jumpToEnter();

				_this2.rl.input.on('keypress', _this2.onType);

				_this2.rl.once('line', function (line) {
					_this2.showCursor();
					_this2.rl.input.removeListener('keypress', _this2.onType);
					_this2.renderAddSuccess(line);
					_this2.startCommit();
				});
			});
		}

		/**
   * Wrapper for jumper.render(). Returns a promise when complete.
   */

	}, {
		key: 'render',
		value: function render(block) {
			return new Promise(function (resolve) {
				jumper.render(block);
				resolve();
			});
		}
	}, {
		key: 'jumpToEnter',
		value: function jumpToEnter() {
			jumper.jumpTo('enter', -1);
			this.rl.setPrompt(this.gitAddPrompt);
		}
	}, {
		key: 'onType',
		value: function onType(char, key) {
			var _this3 = this;

			if (key.name === 'tab') {
				return this.onTab(char, key);
			} else if (['up', 'down'].includes(key.name)) {
				return this.onArrow(char, key);
			} else if (['left', 'right'].includes(key.name)) {
				return;
			} else if (key.name === 'return') {
				// I guess the onType listener hasn't been removed yet
				return;
			}

			var glob = this.rl.line;

			// if the indicator is showing, jump to the enter prompt.
			// this.onType is called after the letter is printed, so immediately
			// "delete" it and then write it on the enter prompt
			if (this.showingIndicator) {
				process.stdout.write(ansiEscapes.cursorBackward(1));
				process.stdout.write(' ');

				jumper.find('enter').content(this.gitAddPrompt);
				this.jumpToEnter();

				process.stdout.write(glob);
				this.showCursor();
				this.fileIndex = 0;
				this.showingIndicator = false;
			}

			jumper.find('enter').content(this.gitAddPrompt + glob);
			this.findGitFiles(glob).then(function () {
				return _this3.render('found');
			}).then(function () {
				return _this3.jumpToEnter();
			});
		}
	}, {
		key: 'onTab',
		value: function onTab(char, key) {
			var _this4 = this;

			if (this.showingDiff) {
				return;
			}

			// the user just typed a tab. We don't want tabs to be printed. What to do?
			// the _deleteLeft method works as a backspace, but _refreshLine is called.
			// this is only bad for when the indicator is showing.

			if (this.showingIndicator) {
				// straight from the source, without _refreshLine()
				this.rl.line = this.rl.line.slice(0, this.rl.cursor - 1) + this.rl.line.slice(this.rl.cursor, this.rl.line.length);
				this.rl.cursor--;
			} else {
				this.rl._deleteLeft();
			}

			this.showingDiff = true;

			var file = this.showingIndicator ? this.getIndicatedFile() : this.rl.line;
			var diff = gitDiffGlob(file, {
				caseSensitive: this.options.caseSensitive
			});

			pager(diff).then(function () {
				return _this4.showingDiff = false;
			});
		}
	}, {
		key: 'onArrow',
		value: function onArrow(char, key) {
			if (!this.showingIndicator && key.name === 'up') {
				return;
			}

			// this is calculated after every keystroke
			if (this.numFiles < 2) {
				return;
			}

			this.prevIndex = this.fileIndex;

			// go to the correct file based on input direction
			if (!this.showingIndicator) {
				this.fileIndex = 0;
				this.hideCursor();
			} else if (key.name === 'up') {
				this.fileIndex -= 1;
			} else if (key.name === 'down') {
				this.fileIndex += 1;
			}

			// if the cursor is at the last file, return
			if (this.fileIndex > this.numFiles - 1) {
				this.fileIndex = this.numFiles - 1;
				return;
			}

			var goToEnter = this.fileIndex < 0;
			this.fileIndex = Math.max(this.fileIndex, 0);

			// at this point the file indicator will change positions and the cursor
			// right next to the printed indicator. So erase it.
			jumper.jumpTo('gitFile' + this.prevIndex, -1);
			process.stdout.write(ansiEscapes.eraseEndLine);

			// if the cursor is moving from the first file up to the prompt area, return
			if (goToEnter) {
				this.fileIndex = 0;
				this.jumpToEnter();
				this.showCursor();
				this.showingIndicator = false;
				return;
			}

			// otherwise, print the indicator at the new position
			jumper.jumpTo('gitFile' + this.fileIndex, -1);
			process.stdout.write(ansiEscapes.cursorForward(2));
			process.stdout.write(chalk.bold.blue('⬅'));

			this.showingIndicator = true;
		}
	}, {
		key: 'showCursor',
		value: function showCursor() {
			process.stdout.write(ansiEscapes.cursorShow);
		}
	}, {
		key: 'hideCursor',
		value: function hideCursor() {
			process.stdout.write(ansiEscapes.cursorHide);
		}
	}, {
		key: 'getIndicatedFile',
		value: function getIndicatedFile() {
			var currentLine = jumper.find('gitFile' + this.fileIndex).escapedText;
			return currentLine.trim().split(' ')[1];
		}
	}, {
		key: 'renderAddSuccess',
		value: function renderAddSuccess(line) {
			// if the indicator is showing, only add that specific file.
			// otherwise, add all files matching the given glob
			if (this.showingIndicator) {
				this.addFiles = [this.getIndicatedFile()];
			} else {
				this.addFiles = this.getGitFilesMatching(line);
			}

			jumper.reset();

			jumper.block(chalk.green('Added files:'), 'found');
			for (var i = 0; i < this.addFiles.length; i++) {
				var file = this.addFiles[i];
				var id = 'addedFile' + i;
				jumper.block(chalk.red(file) + '  ' + chalk.green('✓'), id);
			}
			jumper.break();
			jumper.block('---------------------------------------------');
			jumper.break();
			jumper.block(chalk.green('Enter commit message: "'), 'commit');

			// need to update readline's prompt for _refreshLine calls
			this.rl.setPrompt(jumper.find('commit').text);

			this.render().then(function () {
				jumper.jumpTo('commit', -1);
				process.stdout.write(chalk.green('"'));
				jumper.jumpTo('commit', -1);
			});
		}
	}, {
		key: 'startCommit',
		value: function startCommit() {
			var _this5 = this;

			var onKeypress = function onKeypress() {
				process.stdout.write('' + chalk.green('"'));
				process.stdout.write(ansiEscapes.cursorBackward(1));
			};
			this.rl.input.on('keypress', onKeypress);

			this.rl.once('line', function (line) {
				_this5.commitMessage = line;

				_this5.rl.input.removeListener('keypress', onKeypress);

				var content = jumper.find('commit').text;
				content = '' + content + chalk.green(_this5.commitMessage) + chalk.green('"');
				jumper.find('commit').content(content);

				_this5.commit();
				_this5.complete();
			});

			process.removeListener('exit', this.exitBeforeAdd);
			process.on('exit', this.exitAfterAdd);
		}
	}, {
		key: 'commit',
		value: function commit() {
			process.removeListener('exit', this.exitAfterAdd);

			var _iteratorNormalCompletion2 = true;
			var _didIteratorError2 = false;
			var _iteratorError2 = undefined;

			try {
				for (var _iterator2 = this.addFiles[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
					var file = _step2.value;

					execSync('git add ' + file);
				}
			} catch (err) {
				_didIteratorError2 = true;
				_iteratorError2 = err;
			} finally {
				try {
					if (!_iteratorNormalCompletion2 && _iterator2.return) {
						_iterator2.return();
					}
				} finally {
					if (_didIteratorError2) {
						throw _iteratorError2;
					}
				}
			}

			execSync('git commit -m "' + this.commitMessage + '"');
		}
	}, {
		key: 'complete',
		value: function complete() {

			if (this.options.silent) {
				jumper.erase();
			} else {
				jumper.break();
				jumper.block('---------------------------------------------');
				jumper.break();
				jumper.block('Files committed successfully.');
				jumper.render();
			}

			process.exit();
		}

		/**
   * Finds all files that match the given glob and prints them to the screen,
   * along with their git file type, e.g. 'staged', 'modified', etc.
   * @param {string} [glob] - The glob to match against files.
   */

	}, {
		key: 'findGitFiles',
		value: function findGitFiles(glob) {
			glob = glob || '*';

			// remove previous search
			jumper.removeAllMatching(/gitFile\d+/);

			var fileMap = utils.getFilesOfTypes(this.options.find);
			var types = Object.keys(fileMap);
			var counter = 0;

			var _iteratorNormalCompletion3 = true;
			var _didIteratorError3 = false;
			var _iteratorError3 = undefined;

			try {
				for (var _iterator3 = types[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
					var type = _step3.value;

					var matches = utils.matchGlobsAgainstFiles(fileMap[type], [glob], {
						caseSensitive: this.options.caseSensitive,
						strict: this.options.strict
					});

					var _iteratorNormalCompletion4 = true;
					var _didIteratorError4 = false;
					var _iteratorError4 = undefined;

					try {
						for (var _iterator4 = matches[Symbol.iterator](), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
							var file = _step4.value;

							var text = '  ' + chalk.bold.gray('(' + type + ')') + ' ' + chalk.red(file);
							jumper.block(text, 'gitFile' + counter);
							counter += 1;
						}
					} catch (err) {
						_didIteratorError4 = true;
						_iteratorError4 = err;
					} finally {
						try {
							if (!_iteratorNormalCompletion4 && _iterator4.return) {
								_iterator4.return();
							}
						} finally {
							if (_didIteratorError4) {
								throw _iteratorError4;
							}
						}
					}
				}

				// cache the number of results here, instead of calculating them on every
				// arrow keystroke
			} catch (err) {
				_didIteratorError3 = true;
				_iteratorError3 = err;
			} finally {
				try {
					if (!_iteratorNormalCompletion3 && _iterator3.return) {
						_iterator3.return();
					}
				} finally {
					if (_didIteratorError3) {
						throw _iteratorError3;
					}
				}
			}

			this.numFiles = counter;

			return Promise.resolve();
		}
	}, {
		key: 'getGitFilesMatching',
		value: function getGitFilesMatching(glob) {
			var files = utils.getUniqueFilesOfTypes(this.options.find);
			return utils.matchGlobsAgainstFiles(files, [glob], {
				caseSensitive: this.options.caseSensitive,
				strict: this.options.strict
			});
		}
	}, {
		key: 'exitBeforeAdd',
		value: function exitBeforeAdd() {
			jumper.erase();
		}
	}, {
		key: 'exitAfterAdd',
		value: function exitAfterAdd() {
			process.stdout.write(ansiEscapes.cursorSavePosition);

			for (var i = 0; i < this.addFiles.length; i++) {
				var id = 'addedFile' + i;
				jumper.jumpTo(id, -2);
				process.stdout.write(chalk.red('✗'));
			}

			process.stdout.write(ansiEscapes.cursorRestorePosition);
			console.log();
			console.log();
			console.log('Aborting -- no files added.');
		}
	}]);

	return Interactive;
}();

module.exports = Interactive;